-- 
-- sdram_ctrl.vhd
-- SD-RAM Controller for ESE-MSX
-- 
-- Copyright (c) 2006 Kazuhiro Tsujikawa (ESE Artists' factory)
-- All rights reserved.
-- 
-- Redistribution and use of this source code or any derivative works, are 
-- permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, 
--		this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright 
--		notice, this list of conditions and the following disclaimer in the 
--		documentation and/or other materials provided with the distribution.
-- 3. Redistributions may not be sold, nor may they be used in a commercial 
--		product or activity without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-- EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-- PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
-- OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-- WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-- OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
-- ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- 
-------------------------------------------------------------------------------
-- ALTERA DE1 VERSION

LIBRARY	IEEE;
	USE	IEEE.STD_LOGIC_1164.ALL;
	USE	IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY SDRAMC IS
	PORT(
		MEMCLK			: IN	STD_LOGIC;							-- SD-RAM CLOCK
		-- CPU INTERFACE
		ISLTRFSH_N		: IN	STD_LOGIC;
		WRT_REQ			: IN	STD_LOGIC;
		CPUADR			: IN	STD_LOGIC_VECTOR( 22 DOWNTO 0 );
		DBO				: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		RAMDBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		PAUS			: IN	STD_LOGIC;
		SDPAUS			: OUT	STD_LOGIC;
		-- VDP INTERFACE
		VIDEODLCLK		: IN	STD_LOGIC;
		VIDEODHCLK		: IN	STD_LOGIC;
		WEVDP_N			: IN	STD_LOGIC;
		VDPADR			: IN	STD_LOGIC_VECTOR( 16 DOWNTO 0 );
		VRMDBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		VRMDBI			: OUT	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
		-- SDRAM PORTS
		PMEMCKE			: OUT	STD_LOGIC;							-- SD-RAM CLOCK ENABLE
		PMEMCS_N		: OUT	STD_LOGIC;							-- SD-RAM CHIP SELECT
		PMEMRAS_N		: OUT	STD_LOGIC;							-- SD-RAM ROW/RAS
		PMEMCAS_N		: OUT	STD_LOGIC;							-- SD-RAM /CAS
		PMEMWE_N		: OUT	STD_LOGIC;							-- SD-RAM /WE
		PMEMUDQ			: OUT	STD_LOGIC;							-- SD-RAM UDQM
		PMEMLDQ			: OUT	STD_LOGIC;							-- SD-RAM LDQM
		PMEMBA1			: OUT	STD_LOGIC;							-- SD-RAM BANK SELECT ADDRESS 1
		PMEMBA0			: OUT	STD_LOGIC;							-- SD-RAM BANK SELECT ADDRESS 0
		PMEMADR			: OUT	STD_LOGIC_VECTOR( 11 DOWNTO 0 );	-- SD-RAM ADDRESS
		PMEMDAT			: INOUT	STD_LOGIC_VECTOR( 15 DOWNTO 0 )		-- SD-RAM DATA
	);
END SDRAMC;

ARCHITECTURE RTL OF SDRAMC IS
	SIGNAL FF_SDR_SEQ		: STD_LOGIC_VECTOR(  2 DOWNTO 0 );
	SIGNAL RSTSEQ			: STD_LOGIC_VECTOR(  4 DOWNTO 0 ) := (OTHERS => '0');
	SIGNAL FF_MEM_SEQ		: STD_LOGIC_VECTOR(  1 DOWNTO 0 );
	SIGNAL FREECOUNTER		: STD_LOGIC_VECTOR( 15 DOWNTO 0 ) := (OTHERS => '0');

	SIGNAL SDRSTA			: STD_LOGIC_VECTOR(  2 DOWNTO 0 );
	SIGNAL SDRCMD			: STD_LOGIC_VECTOR(  3 DOWNTO 0 );
	SIGNAL SDRBA			: STD_LOGIC_VECTOR(  1 DOWNTO 0 );
	SIGNAL SDRUDQ			: STD_LOGIC;
	SIGNAL SDRLDQ			: STD_LOGIC;
	SIGNAL SDRADR			: STD_LOGIC_VECTOR( 11 DOWNTO 0 );
	SIGNAL SDRDAT			: STD_LOGIC_VECTOR( 15 DOWNTO 0 );
	SIGNAL FF_SDPAUS		: STD_LOGIC;
	SIGNAL CLRADR			: STD_LOGIC_VECTOR( 17 DOWNTO 0 );
	SIGNAL FF_VDPADR		: STD_LOGIC_VECTOR( 16 DOWNTO 0 );

	CONSTANT SDRCMD_DE		: STD_LOGIC_VECTOR(  3 DOWNTO 0 ) := "1111"; -- DESELECT
	CONSTANT SDRCMD_PR		: STD_LOGIC_VECTOR(  3 DOWNTO 0 ) := "0010"; -- PRECHARGE ALL
	CONSTANT SDRCMD_RE		: STD_LOGIC_VECTOR(  3 DOWNTO 0 ) := "0001"; -- REFRESH
	CONSTANT SDRCMD_MS		: STD_LOGIC_VECTOR(  3 DOWNTO 0 ) := "0000"; -- MODE REGISER SET
	CONSTANT SDRCMD_XX		: STD_LOGIC_VECTOR(  3 DOWNTO 0 ) := "0111"; -- NO OPERATION
	CONSTANT SDRCMD_AC		: STD_LOGIC_VECTOR(  3 DOWNTO 0 ) := "0011"; -- ACTIVATE
	CONSTANT SDRCMD_RD		: STD_LOGIC_VECTOR(  3 DOWNTO 0 ) := "0101"; -- READ
	CONSTANT SDRCMD_WR		: STD_LOGIC_VECTOR(  3 DOWNTO 0 ) := "0100"; -- WRITE
BEGIN
	----------------------------------------------------------------
	-- PORT ASSIGNMENT
	----------------------------------------------------------------
	PMEMCKE		<= '1';
	PMEMCS_N	<= SDRCMD(3);
	PMEMRAS_N	<= SDRCMD(2);
	PMEMCAS_N	<= SDRCMD(1);
	PMEMWE_N	<= SDRCMD(0);

	PMEMUDQ		<= SDRUDQ;
	PMEMLDQ		<= SDRLDQ;
	PMEMBA1		<= SDRBA(1);
	PMEMBA0		<= SDRBA(0);

	PMEMADR		<= SDRADR;
	PMEMDAT		<= SDRDAT;

	SDPAUS		<= FF_SDPAUS;

	----------------------------------------------------------------
	-- RESET CONTROL
	-- "RSTSEQ" SHOULD BE CLEARED WHEN POWER-ON RESET
	----------------------------------------------------------------
	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			FF_MEM_SEQ <= FF_MEM_SEQ(0) & (NOT FF_MEM_SEQ(1));
		END IF;
	END PROCESS;

	PROCESS(MEMCLK)
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( FF_MEM_SEQ = "00" )THEN
				FREECOUNTER <= FREECOUNTER + 1;
			END IF;
		END IF;
	END PROCESS;

	PROCESS(MEMCLK)
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( (FF_MEM_SEQ = "00") AND (FREECOUNTER = X"FFFF") AND (RSTSEQ /= "11111") )THEN
				RSTSEQ <= RSTSEQ + 1;			-- 3MS (= 65536 / 21.48MHZ)
			END IF;
		END IF;
	END PROCESS;

	----------------------------------------------------------------
	-- SD-RAM ACCESS
	----------------------------------------------------------------
	--	 SDRSTA = "000" => IDLE
	--	 SDRSTA = "001" => PRECHARGE ALL
	--	 SDRSTA = "010" => REFRESH
	--	 SDRSTA = "011" => MODE REGISTER SET
	--	 SDRSTA = "100" => READ CPU
	--	 SDRSTA = "101" => WRITE CPU
	--	 SDRSTA = "110" => READ VDP
	--	 SDRSTA = "111" => WRITE VDP
	----------------------------------------------------------------
	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( FF_SDR_SEQ = "111" )THEN
				IF( RSTSEQ(4 DOWNTO 2) = "000" )THEN
					SDRSTA <= "000";												-- IDLE
				ELSIF( RSTSEQ(4 DOWNTO 2) = "001" )THEN
					CASE RSTSEQ(1 DOWNTO 0) IS
						WHEN "00"		=> SDRSTA <= "000";				-- IDLE
						WHEN "01"		=> SDRSTA <= "001";				-- PRECHARGE ALL
						WHEN "10"		=> SDRSTA <= "010";				-- REFRESH (MORE THAN 8 CYCLES)
						WHEN OTHERS		=> SDRSTA <= "011";				-- MODE REGISTER SET
					END CASE;
				ELSIF( RSTSEQ(4 DOWNTO 3) /= "11" )THEN
					SDRSTA <= "101";												-- WRITE (INITIALIZE MEMORY CONTENT)
				ELSIF( VIDEODLCLK = '1' AND (ISLTRFSH_N = '0' OR FF_SDPAUS = '1') )THEN
					SDRSTA <= "010";												-- REFRESH
				ELSE
					-- NORMAL MEMORY ACCESS MODE
					SDRSTA(2) <= '1';												-- READ/WRITE CPU/VDP
				END IF;
			ELSIF( FF_SDR_SEQ = "001" AND SDRSTA(2) = '1' AND RSTSEQ(4 DOWNTO 3) = "11" )THEN
				SDRSTA(1) <= VIDEODLCLK;									-- 0:CPU, 1:VDP
				IF( VIDEODLCLK = '1' )THEN
					--	FOR VDP
					SDRSTA(0) <= NOT WEVDP_N;
				ELSE
					--	FOR CPU
					SDRSTA(0) <= WRT_REQ;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			CASE FF_SDR_SEQ IS
				WHEN "000" =>
					IF( SDRSTA(2) = '1' )THEN				-- CPU/VDP READ/WRITE
						SDRCMD <= SDRCMD_AC;
					ELSIF( SDRSTA(1 DOWNTO 0) = "00" )THEN	-- IDLE
						SDRCMD <= SDRCMD_XX;
					ELSIF( SDRSTA(1 DOWNTO 0) = "01" )THEN	-- PRECHARGE ALL
						SDRCMD <= SDRCMD_PR;
					ELSIF( SDRSTA(1 DOWNTO 0) = "10" )THEN	-- REFRESH
						SDRCMD <= SDRCMD_RE;
					ELSE									-- MODE REGISTER SET
						SDRCMD <= SDRCMD_MS;
					END IF;
				WHEN "001" =>
					SDRCMD <= SDRCMD_XX;
				WHEN "010" =>
					IF( SDRSTA(2) = '1' )THEN
						IF( SDRSTA(0) = '0' )THEN
							SDRCMD <= SDRCMD_RD;			-- "100"(CPU READ) / "110"(VDP READ)
						ELSE
							SDRCMD <= SDRCMD_WR;			-- "101"(CPU WRITE) / "111"(VDP WRITE)
						END IF;
					END IF;
				WHEN "011" =>
					SDRCMD <= SDRCMD_XX;
				WHEN OTHERS	=>
					NULL;
			END CASE;
		END IF;
	END PROCESS;

	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( FF_SDR_SEQ = "000" )THEN
				FF_VDPADR <= VDPADR;
			END IF;
		END IF;
	END PROCESS;

	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			CASE FF_SDR_SEQ IS
				WHEN "000" =>
					SDRUDQ <= '1';
					SDRLDQ <= '1';
				WHEN "010" =>
					IF( SDRSTA(2) = '1' )THEN
						IF( SDRSTA(0) = '0' )THEN
							SDRUDQ <= '0';
							SDRLDQ <= '0';
						ELSE
							IF( RSTSEQ(4 DOWNTO 3) /= "11" )THEN
								SDRUDQ <= '0';
								SDRLDQ <= '0';
							ELSIF( VIDEODLCLK = '0' )THEN
								SDRUDQ <= NOT CPUADR(0);
								SDRLDQ <= CPUADR(0);
							ELSE
								SDRUDQ <= NOT FF_VDPADR(16);
								SDRLDQ <= FF_VDPADR(16);
							END IF;
						END IF;
					END IF;
				WHEN "011" =>
					SDRUDQ <= '1';
					SDRLDQ <= '1';
				WHEN OTHERS
					=> NULL;
			END CASE;
		END IF;
	END PROCESS;

	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			CASE FF_SDR_SEQ IS
			WHEN "000" =>
				IF( SDRSTA(2) = '0' )THEN			-- SET COMMAND MODE
					--	   SINGLE   CL=2 WT=0(SEQ) BL=1
					SDRADR <= "00100" & "010" & "0" & "000";
					SDRBA  <= "00";
				ELSE								-- SET ROW ADDRESS
					SDRBA <= "11";
					IF   ( RSTSEQ(4 DOWNTO 2) = "010" )THEN
						SDRADR <= CLRADR(11 DOWNTO 0);			-- CLEAR VRAM(128KB)
					ELSIF( RSTSEQ(4 DOWNTO 2) = "011" )THEN
						SDRADR <= CLRADR(11 DOWNTO 0);			-- CLEAR ERAM(128KB)
						SDRBA  <= "10";
					ELSIF( RSTSEQ(4 DOWNTO 3) = "10" )THEN
						SDRADR <= CLRADR(11 DOWNTO 0);			-- CLEAR MAINRAM(128KB)
					ELSIF (VIDEODLCLK = '0') THEN
						SDRADR <= CPUADR(12 DOWNTO 1);			-- CPU READ/WRITE
						SDRBA  <= CPUADR(22 DOWNTO 21);
					   ELSE
						SDRADR <= VDPADR(11 DOWNTO 0);			-- VDP READ/WRITE
					END IF;
				END IF;
			WHEN "010" =>										-- SET COLLUMN ADDRESS
				SDRADR(11 DOWNTO 8) <= "0100";					-- A10=1 => ENABLE AUTO PRECHARGE
				SDRBA <= "11";
				-- CLEAR MEMORY
				IF   ( RSTSEQ(4 DOWNTO 2) = "010" )THEN
					SDRADR(7 DOWNTO 0) <= "1000" & CLRADR(15 DOWNTO 12);	-- CLEAR VRAM(128KB)
				ELSIF( RSTSEQ(4 DOWNTO 2) = "011" )THEN
					SDRADR(7 DOWNTO 0) <= "0000" & CLRADR(15 DOWNTO 12);	-- CLEAR ERAM(128KB)
					SDRBA  <= "10";
				ELSIF( RSTSEQ(4 DOWNTO 3) = "10" )THEN
					SDRADR(7 DOWNTO 0) <= "0000" & CLRADR(15 DOWNTO 12);	-- CLEAR MAINRAM(128KB)
				-- WORK MEMORY
				ELSIF( VIDEODLCLK = '0' )THEN
					SDRADR(7 DOWNTO 0) <= CPUADR(20 DOWNTO 13);				-- CPU READ/WRITE
					SDRBA  <= CPUADR(22 DOWNTO 21);
				   ELSE
					SDRADR(7 DOWNTO 0) <= "1000" & VDPADR(15 DOWNTO 12);	-- VDP READ/WRITE
				END IF;
			WHEN OTHERS =>
				NULL;
			END CASE;
		END IF;
	END PROCESS;

	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( FF_SDR_SEQ = "010" )THEN
				IF (SDRSTA(2) = '1') THEN
					IF (SDRSTA(0) = '0') THEN
						SDRDAT <= (OTHERS => 'Z');					-- READ
					ELSE
						IF (RSTSEQ(4 DOWNTO 3) /= "11") THEN
							SDRDAT <= (OTHERS => '0');
						ELSIF( VIDEODLCLK = '0' )THEN
							SDRDAT <= DBO & DBO;					-- "101"(CPU WRITE)
						ELSE
							SDRDAT <= VRMDBO & VRMDBO;				-- "111"(VDP WRITE)
						END IF;
					END IF;
				END IF;
			ELSE
				SDRDAT <= (OTHERS => 'Z');
			END IF;
		END IF;
	END PROCESS;

	---------------------------------------------------------------------------
	-- CLEAR ADDRESS FOR DRAM INITIALIZATION
	---------------------------------------------------------------------------
	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( FF_SDR_SEQ = "010" )THEN
				IF( RSTSEQ(4 DOWNTO 3) /= "11" )THEN
					CLRADR <= (OTHERS => '0');
				ELSE
					CLRADR <= CLRADR + 1;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	---------------------------------------------------------------------------
	-- DATA READ LATCH FOR CPU
	---------------------------------------------------------------------------
	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( FF_SDR_SEQ = "101" )THEN
				IF( SDRSTA = "100" )THEN	-- READ CPU
					IF( CPUADR(0) = '0' )THEN
						RAMDBI	<= PMEMDAT(  7 DOWNTO 0 );
					ELSE
						RAMDBI	<= PMEMDAT( 15 DOWNTO 8 );
					END IF;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	---------------------------------------------------------------------------
	-- DATA READ LATCH FOR VDP
	---------------------------------------------------------------------------
	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( FF_SDR_SEQ = "101" )THEN
				IF( SDRSTA = "110" )THEN	-- READ VDP
					VRMDBI		<= PMEMDAT( 15 DOWNTO 0 );
				END IF;
			END IF;
		END IF;
	END PROCESS;

	---------------------------------------------------------------------------
	-- SDRAM CONTROLLER STATE
	---------------------------------------------------------------------------
	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			IF( FF_SDR_SEQ = "101" )THEN
				IF( SDRSTA(2) = '1' )THEN
					IF( SDRSTA(0) = '0' AND VIDEODLCLK = '0' )THEN
						FF_SDPAUS <= PAUS;
					END IF;
				ELSE
					FF_SDPAUS <= PAUS;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	PROCESS( MEMCLK )
	BEGIN
		IF( MEMCLK'EVENT AND MEMCLK = '1' )THEN
			CASE FF_SDR_SEQ IS
			WHEN "000" =>
				IF( VIDEODHCLK = '1' OR RSTSEQ(4 DOWNTO 3) /= "11" )THEN
					FF_SDR_SEQ <= "001";
				END IF;
			WHEN "111" =>
				IF( VIDEODHCLK = '0' OR RSTSEQ(4 DOWNTO 3) /= "11" )THEN
					FF_SDR_SEQ <= "000";
				END IF;
			WHEN OTHERS =>
				FF_SDR_SEQ <= FF_SDR_SEQ + 1;
			END CASE;
		END IF;
	END PROCESS;
END RTL;
