-- 
-- eseps2.vhd
--   PS/2 keyboard interface for ESE-MSX
--   Revision 1.00
-- 
-- Copyright (c) 2006 Kazuhiro Tsujikawa (ESE Artists' factory)
-- All rights reserved.
-- 
-- Redistribution and use of this source code or any derivative works, are 
-- permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, 
--    this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright 
--    notice, this list of conditions and the following disclaimer in the 
--    documentation and/or other materials provided with the distribution.
-- 3. Redistributions may not be sold, nor may they be used in a commercial 
--    product or activity without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-- EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-- PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
-- OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-- WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-- OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
-- ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
--------------------------------------------------------------------------------
-- Update note
--------------------------------------------------------------------------------
-- Apr 12 2008 - Removed internal INOUT port (modified by t.hara)
-- Oct 05 2006 - Removed 101/106 toggle switch. 
-- Sep 23 2006 - Fixed a problem where some key events are lost after 101/106 
--               keyboard type switching.
-- Sep 22 2006 - Added external default keyboard layout input
-- May 21 2005 - Modified to support Quartus2we5.
-- Jan 24 2004 - Fixed a locking key problem if 101/106 keyboard type is 
--               switched during pressing keys.
--             - Fixed a problem where a comma key is pressed after a 
--               pause key.
-- Jan 23 2004 - Added a 101 keyboard table.
--------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY ESEPS2 IS
	PORT (
		CLK21M		: IN	STD_LOGIC;
		RESET		: IN	STD_LOGIC;
		CLKENA		: IN	STD_LOGIC;

		KMAP		: IN	STD_LOGIC;

		CAPS		: IN	STD_LOGIC;
		KANA		: IN	STD_LOGIC;
		PAUS		: OUT	STD_LOGIC;
		RESO		: OUT	STD_LOGIC;
		
		-- | B7  | B6 | B5   | B4   | B3 | B2  | B1  | B0  |
		-- | SHI | -- | PGUP | PGDN | F9 | F10 | F11 | F12 |
		FKEYS		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

		PPS2CLK		: INOUT	STD_LOGIC;
		PPS2DAT		: INOUT	STD_LOGIC;

		PPIPORTC	: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		PKEYX		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
	);
END ESEPS2;

ARCHITECTURE RTL OF ESEPS2 IS

	SIGNAL FF_PAUS		: STD_LOGIC;
	SIGNAL FF_RESO		: STD_LOGIC;

	SIGNAL SCRO			: STD_LOGIC;
	SIGNAL KEYWE		: STD_LOGIC;
	SIGNAL KEYROW		: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL IKEYCOL		: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL OKEYCOL		: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL MTXIDX		: STD_LOGIC_VECTOR( 10 DOWNTO 0 );
	SIGNAL MTXPTR		: STD_LOGIC_VECTOR(  7 DOWNTO 0 );

	COMPONENT RAM IS
		PORT (
			ADR : IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			CLK : IN	STD_LOGIC;
			WE	: IN	STD_LOGIC;
			DBO : IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			DBI : OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
		);
	END COMPONENT;

	COMPONENT KEYMAP IS
		PORT (
			ADR : IN	STD_LOGIC_VECTOR( 10 DOWNTO 0 );
			CLK : IN	STD_LOGIC;
			DBI : OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
		);
	END COMPONENT;

BEGIN

	PAUS <= FF_PAUS;
	RESO <= FF_RESO;

	PROCESS(CLK21M, RESET, KMAP)

		TYPE TYPPS2SEQ IS (PS2IDLE, PS2RXD, PS2TXD, PS2STOP);
		VARIABLE PS2SEQ : TYPPS2SEQ;
		VARIABLE PS2CHG : STD_LOGIC;
		VARIABLE PS2BRK : STD_LOGIC;
		VARIABLE PS2XE0 : STD_LOGIC;
		VARIABLE PS2XE1 : STD_LOGIC;
		VARIABLE PS2CNT : STD_LOGIC_VECTOR(3 DOWNTO 0);
		VARIABLE PS2CLK : STD_LOGIC_VECTOR(2 DOWNTO 0);
		VARIABLE PS2DAT : STD_LOGIC_VECTOR(7 DOWNTO 0);
		VARIABLE PS2LED : STD_LOGIC_VECTOR(8 DOWNTO 0);
		VARIABLE PS2SKP : STD_LOGIC_VECTOR(2 DOWNTO 0);
		VARIABLE TIMOUT : STD_LOGIC_VECTOR(15 DOWNTO 0);

		VARIABLE PS2CAPS : STD_LOGIC;
		VARIABLE PS2KANA : STD_LOGIC;
		VARIABLE PS2PAUS : STD_LOGIC;
--		VARIABLE PS2SCRO : STD_LOGIC; -- USED FOR 101/106 KEY TABLE SWITCHING
		VARIABLE PS2SHIF : STD_LOGIC; -- REAL SHIFT STATUS 
		VARIABLE PS2VSHI : STD_LOGIC; -- VIRTUAL SHIFT STATUS
		VARIABLE OFKEYS	 : STD_LOGIC_VECTOR(7 DOWNTO 0);		--  LATCH 

		VARIABLE KEYID	 : STD_LOGIC_VECTOR(8 DOWNTO 0);

		TYPE TYPMTXSEQ IS (MTXIDLE, MTXSETTLE, MTXCLEAN, MTXREAD, MTXWRITE, MTXEND, MTXRESET);
		VARIABLE MTXSEQ : TYPMTXSEQ;
		VARIABLE MTXTMP : STD_LOGIC_VECTOR(3 DOWNTO 0);

	BEGIN

		IF (RESET = '1') THEN

			PS2SEQ := PS2IDLE;
			PS2CHG := '0';
			PS2BRK := '0';
			PS2XE0 := '0';
			PS2XE1 := '0';
			PS2CNT := (OTHERS => '0');
			PS2CLK := (OTHERS => '1');
			PS2DAT := (OTHERS => '1');
			TIMOUT := (OTHERS => '1');
			PS2LED := (OTHERS => '1');
			PS2VSHI:= '0';
			PS2SKP := "000";

			PS2CAPS := '1';
			PS2KANA := '1';
			PS2PAUS := '0';
			FF_PAUS		<= '0';
			FF_RESO		<= '0';
			
			OFKEYS	:= (OTHERS=>'0');
			
			SCRO		<= KMAP; -- DEFAULT KEYBOARD LAYOUT / '0': JAPANESE-106, '1': ENGLISH-101

			MTXSEQ := MTXIDLE;

			PPS2DAT <= 'Z';
			PKEYX <= (OTHERS => '1');

			KEYWE		<= '0';
			KEYROW	<= (OTHERS => '0');
			IKEYCOL <= (OTHERS => '0');

		ELSIF (CLK21M'EVENT AND CLK21M = '1') THEN

			IF CLKENA = '1' THEN
				-- "SCAN TABLE > MSX KEY-MATRIX" CONVERSION
				CASE MTXSEQ IS
					WHEN MTXIDLE =>
									 
						IF PS2CHG = '1' THEN 
 
							KEYID := PS2XE0 & PS2DAT;
							IF SCRO = '1' THEN
								MTXSEQ := MTXSETTLE;
								MTXIDX <= "0" & (NOT PS2SHIF) & KEYID;
							ELSE
								MTXSEQ := MTXREAD;
								MTXIDX <= "10" & KEYID;
							END IF;
							PKEYX <= (OTHERS => '1');
				
						ELSE
				
							FOR I IN 7 DOWNTO 1 LOOP
								IF OKEYCOL(I) = '1' THEN
									PKEYX(I) <= '0';
								ELSE
									PKEYX(I) <= '1';
								END IF;
							END LOOP;
							IF PPIPORTC(3 DOWNTO 0) = "0110" THEN
								IF (SCRO = '0' AND OKEYCOL(0) = '1') OR (SCRO = '1' AND PS2VSHI = '1') THEN
									PKEYX(0) <= '0';
								ELSE
									PKEYX(0) <= '1';
								END IF;
							ELSE
								IF OKEYCOL(0) = '1' THEN
									PKEYX(0) <= '0';
								ELSE
									PKEYX(0) <= '1';
								END IF;
							END IF;
							KEYROW <= "0000" & PPIPORTC(3 DOWNTO 0);
						END IF;
					
					WHEN MTXSETTLE =>
						MTXSEQ := MTXCLEAN;
						KEYWE	 <= '0';
						KEYROW <= "0000" & MTXPTR(3 DOWNTO 0);
		 
					WHEN MTXCLEAN =>													
						MTXSEQ := MTXREAD;
						KEYWE <= '1';
						IKEYCOL <= OKEYCOL;
						IKEYCOL(CONV_INTEGER(MTXPTR(6 DOWNTO 4))) <= '0';
						MTXIDX <= "0" & PS2SHIF & KEYID;
					
					WHEN MTXREAD =>
						MTXSEQ := MTXWRITE;
						KEYWE <= '0';
						KEYROW <= "0000" & MTXPTR(3 DOWNTO 0);
						IF( PS2BRK = '0' ) THEN 
							PS2VSHI := MTXPTR(7);
						ELSE
							PS2VSHI := PS2SHIF;
						END IF;

					WHEN MTXWRITE	 =>
						MTXSEQ := MTXEND;
						KEYWE <= '1';
						IKEYCOL <= OKEYCOL;
						IKEYCOL(CONV_INTEGER(MTXPTR(6 DOWNTO 4))) <= NOT PS2BRK;

					WHEN MTXEND	 =>
						MTXSEQ := MTXIDLE;
						KEYWE <= '0';
						KEYROW <= "0000" & PPIPORTC(3 DOWNTO 0);
						PS2CHG := '0';
						PS2BRK := '0';
						PS2XE0 := '0';
						PS2XE1 := '0';

					WHEN MTXRESET =>
						IF MTXTMP = "1011" THEN
							MTXSEQ := MTXIDLE;
							KEYWE <= '0';
							KEYROW <= "0000" & PPIPORTC(3 DOWNTO 0);
						END IF;
						KEYWE		<= '1';
						KEYROW	<= "0000" & MTXTMP;
						IKEYCOL <= (OTHERS => '0');
						MTXTMP := MTXTMP + '1';

					WHEN OTHERS =>
						MTXSEQ := MTXIDLE;

				END CASE;

			END IF;

			-- "PS/2 INTERFACE > SCAN TABLE" CONVERSION
			IF (CLKENA = '1') THEN		-- CLKENA = 3.579MHZ ŔpX, 1[S]/3.579[MHZ] = 279[NS]

				IF (PS2CLK = "100") THEN				-- CLK INACTIVE	ǧ̂k
					PS2CLK(2) := '0';
					TIMOUT := X"01FF";						-- COUNTDOWN TIMEOUT (143US = 279NS X 512CLK, EXCEED 100US)

					IF (PS2SEQ = PS2IDLE) THEN	--	AChXe[g
						PPS2DAT <= 'Z';
						PS2SEQ := PS2RXD;			--	MXe[g֑J
						PS2CNT := (OTHERS => '0');

					ELSIF (PS2SEQ = PS2TXD) THEN	--	MXe[g
						IF (PS2CNT = "1000") THEN	--	8BITMI
							PS2CAPS := CAPS;
							PS2KANA := KANA;
							PS2PAUS := FF_PAUS;
--							PS2SCRO := SCRO;
							PS2SEQ := PS2IDLE;		-- AChXe[g֑J
						END IF;
						PPS2DAT <= PS2LED(0);		-- LSB-FIRSTőM
						PS2LED := PS2LED(0) & PS2LED(8 DOWNTO 1);	-- Mrbg MSB ֈړirbg[eCgj
						PS2DAT := '1' & PS2DAT(7 DOWNTO 1);
						PS2CNT := PS2CNT + 1;		-- Mrbg̐XV

					ELSIF (PS2SEQ = PS2RXD) THEN	--	MXe[g
						IF (PS2CNT = "0111") THEN
							PS2SEQ := PS2STOP;		-- XgbvXe[g֑J
						END IF;
						PS2DAT := PPS2DAT & PS2DAT(7 DOWNTO 1);
						PS2CNT := PS2CNT + 1;

					ELSIF (PS2SEQ = PS2STOP) THEN	-- XgbvXe[giMXe[gHj
						PS2SEQ := PS2IDLE;
			-- MR[h𔻒f
						IF (PS2DAT = X"AA") THEN		-- BAT CODE (BASIC ASSURANCE TEST)
							PS2CAPS := NOT CAPS;
							PS2KANA := NOT KANA;
							PS2PAUS := NOT FF_PAUS;
--							PS2SCRO := NOT SCRO;
						ELSIF PS2SKP /= "000" THEN	-- SKIP SOME SEQUENCES.
							PS2SKP := PS2SKP - 1;
						ELSIF (PS2DAT = X"14" AND PS2XE0 = '0' AND PS2XE1 = '1') THEN -- PAUSE/BREAK MAKE
							IF PS2BRK ='0' THEN
								FF_PAUS <= NOT FF_PAUS;	 -- CPU PAUSE
								PS2SKP := "110";	 -- SKIP THE NEXT 6 SEQUENCES.
							END IF;
						ELSIF (PS2DAT = X"7C" AND PS2XE0 = '1' AND PS2XE1 = '0') THEN -- PRINTSCREEN MAKE
							IF PS2BRK = '0' THEN
								FF_RESO <= NOT FF_RESO;	 -- TOGGLE DISPLAY MODE
							END IF;
							PS2CHG := '1';
						ELSIF (PS2DAT = X"7D" AND PS2XE0 = '1' AND PS2XE1 = '0') THEN -- PGUP MAKE
							IF PS2BRK = '0' THEN
								OFKEYS(5) := NOT OFKEYS(5);
							END IF;
							PS2CHG := '1';
						ELSIF (PS2DAT = X"7A" AND PS2XE0 = '1' AND PS2XE1 = '0') THEN -- PGDN MAKE
							IF PS2BRK = '0' THEN
								OFKEYS(4) := NOT OFKEYS(4);
							END IF;
							PS2CHG := '1';
						ELSIF (PS2DAT = X"01" AND PS2XE0 = '0' AND PS2XE1 = '0') THEN -- F9 MAKE
							IF PS2BRK = '0' THEN
								OFKEYS(3) := NOT OFKEYS(3);
							END IF;
							PS2CHG := '1';
						ELSIF (PS2DAT = X"09" AND PS2XE0 = '0' AND PS2XE1 = '0') THEN -- F10 MAKE
							IF PS2BRK = '0' THEN
								OFKEYS(2) := NOT OFKEYS(2);
							END IF;
							PS2CHG := '1';
						ELSIF (PS2DAT = X"78" AND PS2XE0 = '0' AND PS2XE1 = '0') THEN -- F11 MAKE
							IF PS2BRK = '0' THEN
								OFKEYS(1) := NOT OFKEYS(1);
							END IF;
							PS2CHG := '1';
						ELSIF (PS2DAT = X"07" AND PS2XE0 = '0' AND PS2XE1 = '0') THEN -- F12 MAKE
							IF PS2BRK = '0' THEN
								OFKEYS(0) := NOT OFKEYS(0);	 -- TOGGLE ONSCREENDISPLAY ENABLE.
							END IF;
							PS2CHG := '1';
						ELSIF ((PS2DAT = X"12" OR PS2DAT = X"59") AND PS2XE0 = '0' AND PS2XE1 ='0' ) THEN -- SHIFT MAKE
							PS2SHIF:= NOT PS2BRK;
							OFKEYS(7) := PS2SHIF;
							PS2CHG := '1';
						ELSIF (PS2DAT = X"F0") THEN -- BREAK CODE	uCNR[hEvtBbNX
							PS2BRK := '1';
						ELSIF (PS2DAT = X"E0") THEN -- EXTND CODE E0
							PS2XE0 := '1';
						ELSIF (PS2DAT = X"E1") THEN -- EXTND CODE E1 (IGNORE)
							PS2XE1 := '1';
						ELSIF (PS2DAT = X"FA") THEN	 -- ACK OF "EDH" COMMAND	ACK
							PS2SEQ := PS2IDLE;	-- AChԂ֑J
						ELSE
							PS2CHG := '1';
						END IF;

					END IF;

				ELSIF (PS2CLK = "011") THEN			-- CLK ACTIVE	gĂ
					PS2CLK(2) := '1';
					TIMOUT := X"01FF";						-- COUNTDOWN TIMEOUT (143US = 279NS X 512CLK, EXCEED 100US)

				ELSIF (TIMOUT = X"0000") THEN		-- ^CAEg

					PPS2DAT <= 'Z';
					PS2SEQ := PS2IDLE;						-- TO IDLE STATE

					IF (PS2SEQ = PS2IDLE AND PS2CLK(2) = '1') THEN

						IF (PS2DAT = X"FA" AND PS2LED = "111101101") THEN	-- ȎMf[^ [ED](ZbgEXe[^XECWP[^)
																			-- ŁAMf[^ [FA](ACK) ̏ꍇ

				-- [EDH][ACK][OPT][ACK]
							PS2SEQ := PS2TXD;					-- MXe[g֑J
							PPS2DAT <= '0';			-- X^[grbg

							PS2LED := (CAPS XOR KANA XOR FF_PAUS XOR '1') & "00000" & (NOT CAPS) & FF_PAUS & (NOT KANA);
							TIMOUT := X"FFFF";				-- COUNTDOWN TIMEOUT (18.3MS = 279NS X 65536CLK, EXCEED 1MS)

						ELSIF (CAPS /= PS2CAPS OR KANA /= PS2KANA OR FF_PAUS /= PS2PAUS) THEN -- CWP[^ωH
							PS2SEQ := PS2TXD;					-- MXe[g֑J
							PPS2DAT <= '0';			-- X^[grbg

							PS2LED := "111101101";		-- COMMAND EDH Xe[^XECWP[^
							TIMOUT := X"FFFF";				-- COUNTDOWN TIMEOUT (18.3MS = 279NS X 65536CLK, EXCEED 1MS)

						END IF;
					END IF;

				ELSE
					TIMOUT := TIMOUT - 1;					-- COUNTDOWN TIMEOUT

				END IF;

				PS2CLK(1) := PS2CLK(0);
				PS2CLK(0) := PPS2CLK;

			END IF;

		END IF;

		FKEYS <= OFKEYS;

	END PROCESS;

	PPS2CLK <= 'Z';

	U1: RAM
	PORT MAP (
		ADR => KEYROW	,
		CLK => CLK21M	,
		WE	=> KEYWE	,
		DBO => IKEYCOL	,
		DBI => OKEYCOL	
	);

	U2: KEYMAP
	PORT MAP (
		ADR => MTXIDX	,
		CLK => CLK21M	,
		DBI => MTXPTR	
	);

END RTL;
